/******************************************************************************
 *
 * Copyright(c) 2007 - 2017 Realtek Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 *****************************************************************************/


#ifndef __NIC_SPEC_H__
#define __NIC_SPEC_H__

#include <drv_conf.h>

#define RTL8711_MCTRL_		(0x20000)
#define RTL8711_UART_		(0x30000)
#define RTL8711_TIMER_		(0x40000)
#define RTL8711_FINT_		(0x50000)
#define RTL8711_HINT_		(0x50000)
#define RTL8711_GPIO_		(0x60000)
#define RTL8711_WLANCTRL_	(0x200000)
#define RTL8711_WLANFF_		(0xe00000)
#define RTL8711_HCICTRL_	(0x600000)
#define RTL8711_SYSCFG_		(0x620000)
#define RTL8711_SYSCTRL_	(0x620000)
#define RTL8711_MCCTRL_		(0x020000)


#include <rtl8711_regdef.h>

#include <rtl8711_bitdef.h>


#endif /* __RTL8711_SPEC_H__ */
