/******************************************************************************
 *
 * Copyright(c) 2016 - 2019 Realtek Corporation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 ******************************************************************************/

#ifndef __HALMAC_COM_REG_H__
#define __HALMAC_COM_REG_H__

#include "halmac_hw_cfg.h"

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_ISO_CTRL 0x0000

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_TX_CTRL 0x10250000

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_FUNC_EN 0x0002
#define REG_SYS_PW_CTRL 0x0004

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_CMD11_VOL_SWITCH 0x10250004
#define REG_SDIO_CTRL 0x10250005
#define REG_SDIO_DRIVING 0x10250006

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_CLK_CTRL 0x0008

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_MONITOR 0x10250008

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_EEPROM_CTRL 0x000A
#define REG_EE_VPD 0x000C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_MONITOR_2 0x1025000C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_SWR_CTRL1 0x0010

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_SDIO_CTRL_2 0x10250010

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_SWR_CTRL2 0x0014

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_HIMR 0x10250014

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_SWR_CTRL3 0x0018

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_HISR 0x10250018

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RSV_CTRL 0x001C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_RX_REQ_LEN 0x1025001C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RF_CTRL 0x001F

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_RF0_CTRL 0x001F

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_FREE_TXPG_SEQ_V1 0x1025001F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_AFE_LDO_CTRL 0x0020

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_FREE_TXPG 0x10250020

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_AFE_CTRL1 0x0024

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_FREE_TXPG2 0x10250024

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8881A_SUPPORT)

#define REG_AFE_CTRL2 0x0028

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_OQT_FREE_TXPG_V1 0x10250028

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPARSW_POW_MAC 0x0028
#define REG_ANAPARLDO_POW_MAC 0x0029
#define REG_ANAPAR_POW_MAC 0x002A
#define REG_ANAPAR_POW_XTAL 0x002B

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8881A_SUPPORT)

#define REG_AFE_CTRL3 0x002C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPARLDO_MAC 0x002C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_TXPKT_EMPTY 0x1025002C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_EFUSE_CTRL 0x0030

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_HTSFR_INFO 0x10250030

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_LDO_EFUSE_CTRL 0x0034
#define REG_PWR_OPTION_CTRL 0x0038

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_HCPWM1_V2 0x10250038
#define REG_SDIO_HCPWM2_V2 0x1025003A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_CAL_TIMER 0x003C
#define REG_ACLK_MON 0x003E

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_GPIO_MUXCFG_2 0x003F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_GPIO_MUXCFG 0x0040

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_INDIRECT_REG_CFG 0x10250040

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_GPIO_PIN_CTRL 0x0044

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_INDIRECT_REG_DATA 0x10250044

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_GPIO_INTM 0x0048
#define REG_LED_CFG 0x004C
#define REG_FSIMR 0x0050
#define REG_FSISR 0x0054
#define REG_HSIMR 0x0058
#define REG_HSISR 0x005C
#define REG_GPIO_EXT_CTRL 0x0060

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_H2C 0x10250060

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PAD_CTRL1 0x0064

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_C2H 0x10250064

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_WL_BT_PWR_CTRL 0x0068

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_SDM_DEBUG 0x006C

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_GSSR 0x006C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_SYS_SDIO_CTRL 0x0070

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_SYS_CLKR 0x0070

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_HCI_OPT_CTRL 0x0074

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_AFE_CTRL4 0x0078

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HCI_BG_CTRL 0x0078

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_AFE_XTAL_CTRL_EXT 0x0078

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HCI_LDO_CTRL 0x007A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_LDO_SWR_CTRL 0x007C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_8051FW_CTRL 0x0080

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MCUFW_CTRL 0x0080

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_HRPWM1 0x10250080
#define REG_SDIO_HRPWM2 0x10250082

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_MCU_TST_CFG 0x0084

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_HPS_CLKR 0x10250084
#define REG_SDIO_BUS_CTRL 0x10250085

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_HSUS_CTRL 0x10250086

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_HMEBOX_E0_E1 0x0088

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_RESPONSE_TIMER 0x10250088

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_CMD_CRC 0x1025008A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_HMEBOX_E2_E3 0x008C
#define REG_WLLPS_CTRL 0x0090

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_HSISR 0x10250090

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_HSIMR 0x10250091

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT)

#define REG_AFE_CTRL5 0x0094

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_GPIO_DEBOUNCE_CTRL 0x0098
#define REG_RPWM2 0x009C
#define REG_SYSON_FSM_MON 0x00A0

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT)

#define REG_AFE_CTRL6 0x00A4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PMC_DBG_CTRL1 0x00A8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT)

#define REG_AFE_CTRL7 0x00AC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_HIMR0 0x00B0
#define REG_HISR0 0x00B4
#define REG_HIMR1 0x00B8
#define REG_HISR1 0x00BC
#define REG_DBG_PORT_SEL 0x00C0

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_DIOERR_RPT 0x102500C0

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_SDIO_ERR_RPT 0x102500C0

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_CMD_ERRCNT 0x102500C2
#define REG_SDIO_DATA_ERRCNT 0x102500C3

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PAD_CTRL2 0x00C4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_CMD_ERR_CONTENT 0x102500C4

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_MEM_RMC 0x00C8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SDIO_CRC_ERR_IDX 0x102500C9

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_DATA_CRC 0x102500CA

#endif

#if (HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_SDIO_DATA_REPLY_TIME 0x102500CB

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PMC_DBG_CTRL2 0x00CC

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SDIO_TRANS_FIFO_STATUS 0x102500CC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BIST_CTRL 0x00D0
#define REG_BIST_RPT 0x00D4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_MEM_CTRL 0x00D8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_AFE_CTRL8 0x00DC

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_WLAN_DBG 0x00DC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_SYN_RFC_CTRL 0x00DC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_USB_SIE_INTF 0x00E0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_SYS_PINMUX 0x00E0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PCIE_MIO_INTF 0x00E4
#define REG_PCIE_MIO_INTD 0x00E8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_WLRF1 0x00EC

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_HPON_FSM 0x00EC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SYS_CFG1 0x00F0
#define REG_SYS_STATUS1 0x00F4
#define REG_SYS_STATUS2 0x00F8
#define REG_SYS_CFG2 0x00FC
#define REG_CR 0x0100

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PG_SIZE 0x0104

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PKT_BUFF_ACCESS_CTRL 0x0106

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TSF_CLK_STATE 0x0108
#define REG_TXDMA_PQ_MAP 0x010C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TRXFF_BNDY 0x0114

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RXFF_BNDY_V1 0x0114

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PTA_I2C_MBOX 0x0118

#endif

#if (HALMAC_8814A_SUPPORT)

#define REG_FF_STATUS 0x0118

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RXFF_PTR 0x011C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXFF_BNDY 0x011C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_FEIMR 0x0120

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FE1IMR 0x0120

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_FEISR 0x0124

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FE1ISR 0x0124

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_CPWM 0x012C
#define REG_FWIMR 0x0130
#define REG_FWISR 0x0134
#define REG_FTIMR 0x0138
#define REG_FTISR 0x013C
#define REG_PKTBUF_DBG_CTRL 0x0140
#define REG_PKTBUF_DBG_DATA_L 0x0144
#define REG_PKTBUF_DBG_DATA_H 0x0148
#define REG_CPWM2 0x014C
#define REG_TC0_CTRL 0x0150
#define REG_TC1_CTRL 0x0154
#define REG_TC2_CTRL 0x0158
#define REG_TC3_CTRL 0x015C
#define REG_TC4_CTRL 0x0160
#define REG_TCUNIT_BASE 0x0164
#define REG_TC5_CTRL 0x0168
#define REG_TC6_CTRL 0x016C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBIST_FAIL 0x0170

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_MBIST_DRF_FAIL 0x0170

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBIST_START_PAUSE 0x0174
#define REG_MBIST_DONE 0x0178

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBIST_ROM_CRC_DATA 0x017C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MBIST_NRML_FAIL 0x017C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_MBIST_FAIL_NRML 0x017C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MBIST_READ_BIST_RPT 0x017C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_AES_DECRPT_DATA 0x0180
#define REG_AES_DECRPT_CFG 0x0184

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MBIST_READ_BIST_RPT_V1 0x0188

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HIOE_CTRL 0x0188

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT)

#define REG_MACCLKFRQ 0x018C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HIOE_CFG_FILE 0x018C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TMETER 0x0190
#define REG_OSC_32K_CTRL 0x0194

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_32K_CAL_REG1 0x0198

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_32K_CAL_REG0 0x0198

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_C2HEVT 0x01A0

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_C2HEVT_1 0x01A4
#define REG_C2HEVT_2 0x01A8
#define REG_C2HEVT_3 0x01AC

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MISC_CTRL_V1 0x01B0

#endif

#if (HALMAC_8814A_SUPPORT)

#define REG_TC7_CTRL 0x01B0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RXDESC_BUFF_RPTR 0x01B0

#endif

#if (HALMAC_8814A_SUPPORT)

#define REG_TC8_CTRL 0x01B4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RXDESC_BUFF_WPTR 0x01B4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SW_DEFINED_PAGE1 0x01B8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SW_DEFINED_PAGE2 0x01BC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_MCUTST_I 0x01C0
#define REG_MCUTST_II 0x01C4
#define REG_FMETHR 0x01C8
#define REG_HMETFR 0x01CC
#define REG_HMEBOX0 0x01D0
#define REG_HMEBOX1 0x01D4
#define REG_HMEBOX2 0x01D8
#define REG_HMEBOX3 0x01DC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_LLT_INIT 0x01E0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_LLT_IND_ACCESS 0x01E0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RXDESC_BUFF_BNDY 0x01E0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_GENTST 0x01E4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_LLT_INIT_ADDR 0x01E4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_BB_ACCESS_CTRL 0x01E8
#define REG_BB_ACCESS_DATA 0x01EC
#define REG_HMEBOX_E0 0x01F0
#define REG_HMEBOX_E1 0x01F4
#define REG_HMEBOX_E2 0x01F8
#define REG_HMEBOX_E3 0x01FC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RQPN_CTRL_HLPQ 0x0200

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_CTRL_1 0x0200

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCN_CTRL_0 0x0200

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_FIFOPAGE_INFO 0x0204

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_CTRL_2 0x0204

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCN_CTRL_1 0x0204

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_DWBCN0_CTRL 0x0208

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_AUTO_LLT_V1 0x0208

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TXDMA_OFFSET_CHK 0x020C
#define REG_TXDMA_STATUS 0x0210

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RQPN_NPQ 0x0214

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_TX_DMA_DBG 0x0214

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TQPNT1 0x0218

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DMA_RQPN_INFO_PUB 0x0218

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TQPNT2 0x021C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RQPN_CTRL_2_V1 0x021C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TDE_DEBUG 0x0220

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TQPNT3 0x0220

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCN_CTRL_2 0x0220

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_AUTO_LLT 0x0224

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TQPNT4 0x0224

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_DWBCN1_CTRL 0x0228

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RQPN_CTRL_1 0x0228
#define REG_RQPN_CTRL_2 0x022C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RQPN_EXQ1_EXQ2 0x0230

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_INFO_1 0x0230

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXPKTNUM_0 0x0230

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TQPNT3_V1 0x0234

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_INFO_2 0x0234

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXPKTNUM_1 0x0234

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_INFO_3 0x0238

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXPKTNUM_2 0x0238

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_INFO_4 0x023C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXPKTNUM_3 0x023C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FIFOPAGE_INFO_5 0x0240

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TX_AGG_ALIGN 0x0240

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_H2C_HEAD 0x0244
#define REG_H2C_TAIL 0x0248
#define REG_H2C_READ_ADDR 0x024C
#define REG_H2C_WR_ADDR 0x0250
#define REG_H2C_INFO 0x0254

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_FIFOPAGE_CTRL_5 0x0258
#define REG_FIFOPAGE_CTRL_3 0x025C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TQPNT5 0x0260

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DMA_OQT_0 0x0260

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TQPNT6 0x0264

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DMA_OQT_1 0x0264

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_FIFOPAGE_INFO_6 0x0268
#define REG_FIFOPAGE_INFO_7 0x026C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PGSUB_CNT 0x026C

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_FIFOPAGE_CTRL_4 0x0270

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PGSUB_H 0x0270
#define REG_PGSUB_N 0x0274
#define REG_PGSUB_L 0x0278
#define REG_PGSUB_E 0x027C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RXDMA_AGG_PG_TH 0x0280

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RXPKT_NUM 0x0284

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RXDMA_CTRL 0x0284

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RXDMA_STATUS 0x0288
#define REG_RXDMA_DPR 0x028C
#define REG_RXDMA_MODE 0x0290
#define REG_C2H_PKT 0x0294

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FWFF_C2H 0x0298
#define REG_FWFF_CTRL 0x029C
#define REG_FWFF_PKT_INFO 0x02A0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_FC2H_INFO 0x02A4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWFF_PKT_INFO2 0x02A4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXPKTNUM 0x02B0
#define REG_RXPKTNUM_TH 0x02B4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FW_UPD_RXDES_RDPTR 0x02B8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FW_MSG1 0x02E0
#define REG_FW_MSG2 0x02E4
#define REG_FW_MSG3 0x02E8
#define REG_FW_MSG4 0x02EC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_CTRL 0x0300

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_CTRL 0x0300

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_LX_CTRL1 0x0300

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_INT_MIG 0x0304

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH_CTRL 0x0304

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BCNQ_TXBD_DESA 0x0308

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HIQ_CTRL 0x0308
#define REG_INT_MIG_V1 0x030C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MGQ_TXBD_DESA 0x0310

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0MGQ_TXBD_DESA_L 0x0310
#define REG_P0MGQ_TXBD_DESA_H 0x0314

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_VOQ_TXBD_DESA 0x0318

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH0_TXBD_DESA_L 0x0318
#define REG_ACH0_TXBD_DESA_H 0x031C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_VIQ_TXBD_DESA 0x0320

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH1_TXBD_DESA_L 0x0320
#define REG_ACH1_TXBD_DESA_H 0x0324

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BEQ_TXBD_DESA 0x0328

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH2_TXBD_DESA_L 0x0328
#define REG_ACH2_TXBD_DESA_H 0x032C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BKQ_TXBD_DESA 0x0330

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH3_TXBD_DESA_L 0x0330
#define REG_ACH3_TXBD_DESA_H 0x0334

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RXQ_RXBD_DESA 0x0338

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0RXQ_RXBD_DESA_L 0x0338
#define REG_P0RXQ_RXBD_DESA_H 0x033C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI0Q_TXBD_DESA 0x0340

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0BCNQ_TXBD_DESA_L 0x0340
#define REG_P0BCNQ_TXBD_DESA_H 0x0344

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI1Q_TXBD_DESA 0x0348

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWCMDQ_TXBD_DESA_L 0x0348
#define REG_FWCMDQ_TXBD_DESA_H 0x034C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI2Q_TXBD_DESA 0x0350

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_HRPWM1_HCPWM1_DCPU 0x0354

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI3Q_TXBD_DESA 0x0358

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0_MPRT_BCNQ_TXBD_DESA_L 0x0358
#define REG_P0_MPRT_BCNQ_TXBD_DESA_H 0x035C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI4Q_TXBD_DESA 0x0360
#define REG_HI5Q_TXBD_DESA 0x0368

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0_MPRT_BCNQ_TXRXBD_NUM 0x036C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI6Q_TXBD_DESA 0x0370

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BD_RWPTR_CLR2 0x0370
#define REG_BD_RWPTR_CLR3 0x0374

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI7Q_TXBD_DESA 0x0378

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0MGQ_RXQ_TXRXBD_NUM 0x0378
#define REG_CHNL_DMA_CFG 0x037C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MGQ_TXBD_NUM 0x0380

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWCMDQ_TXBD_NUM 0x0380

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RX_RXBD_NUM 0x0382
#define REG_VOQ_TXBD_NUM 0x0384

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH0_ACH1_TXBD_NUM 0x0384

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_VIQ_TXBD_NUM 0x0386
#define REG_BEQ_TXBD_NUM 0x0388

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH2_ACH3_TXBD_NUM 0x0388

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BKQ_TXBD_NUM 0x038A
#define REG_HI0Q_TXBD_NUM 0x038C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI0Q_HI1Q_TXBD_NUM 0x038C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI1Q_TXBD_NUM 0x038E
#define REG_HI2Q_TXBD_NUM 0x0390

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI2Q_HI3Q_TXBD_NUM 0x0390

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI3Q_TXBD_NUM 0x0392
#define REG_HI4Q_TXBD_NUM 0x0394

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI4Q_HI5Q_TXBD_NUM 0x0394

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI5Q_TXBD_NUM 0x0396
#define REG_HI6Q_TXBD_NUM 0x0398

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI6Q_HI7Q_TXBD_NUM 0x0398

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI7Q_TXBD_NUM 0x039A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TSFTIMER_HCI 0x039C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BD_RWPTR_CLR 0x039C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BD_RWPTR_CLR1 0x039C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_VOQ_TXBD_IDX 0x03A0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH0_TXBD_IDX 0x03A0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_VIQ_TXBD_IDX 0x03A4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH1_TXBD_IDX 0x03A4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BEQ_TXBD_IDX 0x03A8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH2_TXBD_IDX 0x03A8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BKQ_TXBD_IDX 0x03AC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH3_TXBD_IDX 0x03AC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MGQ_TXBD_IDX 0x03B0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0MGQ_TXBD_IDX 0x03B0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RXQ_RXBD_IDX 0x03B4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0RXQ_RXBD_IDX 0x03B4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI0Q_TXBD_IDX 0x03B8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI0Q_TXBD_IDX 0x03B8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI1Q_TXBD_IDX 0x03BC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI1Q_TXBD_IDX 0x03BC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI2Q_TXBD_IDX 0x03C0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI2Q_TXBD_IDX 0x03C0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI3Q_TXBD_IDX 0x03C4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI3Q_TXBD_IDX 0x03C4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI4Q_TXBD_IDX 0x03C8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI4Q_TXBD_IDX 0x03C8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI5Q_TXBD_IDX 0x03CC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI5Q_TXBD_IDX 0x03CC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI6Q_TXBD_IDX 0x03D0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI6Q_TXBD_IDX 0x03D0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HI7Q_TXBD_IDX 0x03D4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI7Q_TXBD_IDX 0x03D4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DBG_SEL_V1 0x03D8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DBGSEL_PCIE_HRPWM1_HCPWM1_V1 0x03D8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PCIE_HRPWM1_V1 0x03D9

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_HRPWM1_V1 0x03D9

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PCIE_HCPWM1_V1 0x03DA

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_HCPWM1_V1 0x03DA

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_CTRL2 0x03DB

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_CTRL2 0x03DB

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_LX_CTRL2 0x03DB

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PCIE_HRPWM2_V1 0x03DC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_HRPWM2_V1 0x03DC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_HRPWM2_HCPWM2_V1 0x03DC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PCIE_HCPWM2_V1 0x03DE

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_HCPWM2_V1 0x03DE

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PCIE_H2C_MSG_V1 0x03E0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_H2C_MSG_V1 0x03E0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PCIE_C2H_MSG_V1 0x03E4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HCI_C2H_MSG_V1 0x03E4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DBI_WDATA_V1 0x03E8

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_LX_DMA_ISR 0x03E8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DBI_RDATA_V1 0x03EC

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_LX_DMA_IMR 0x03EC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DBI_FLAG_V1 0x03F0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_STUCK_FLAG_V1 0x03F0

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_LX_DMA_DBG 0x03F0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MDIO_V1 0x03F4

#endif

#if (HALMAC_8192E_SUPPORT)

#define REG_MDIO2_V1 0x03F8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_MIX_CFG 0x03F8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_WDT_CFG 0x03F8

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_BUS_MIX_CFG 0x03F8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HCI_MIX_CFG 0x03FC

#endif

#if (HALMAC_8881A_SUPPORT)

#define REG_BUS_MIX_CFG1 0x03FC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_Q0_INFO 0x0400

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_QUEUELIST_INFO0 0x0400

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_QUEUE_INFO1 0x0400

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_Q1_INFO 0x0404

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_QUEUELIST_INFO1 0x0404

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_QUEUE_INFO2 0x0404

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_Q2_INFO 0x0408

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_QUEUE_INFO3 0x0408

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_QUEUELIST_INFO2 0x0408

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_Q3_INFO 0x040C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_QINFO_INDEX 0x040C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_QUEUELIST_INFO3 0x040C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MGQ_INFO 0x0410

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_QUEUE_EMPTY 0x0410

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_QUEUELIST_INFO_EMPTY 0x0410

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HIQ_INFO 0x0414

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_QUEUELIST_INFO2_V1 0x0414

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ACQ_STOP_V1 0x0414

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_QUEUELIST_ACQ_EN 0x0414

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BCNQ_INFO 0x0418

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TXPKT_EMPTY_V1 0x0418

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCNQ_BDNY_V2 0x0418

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TXPKT_EMPTY 0x041A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_CPU_MGQ_INFO 0x041C
#define REG_FWHW_TXQ_CTRL 0x0420

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HWSEQ_CTRL 0x0423

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_DATAFB_SEL 0x0423

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BCNQ_BDNY 0x0424

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BCNQ_BDNY_V1 0x0424

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXBDNY 0x0424

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MGQ_BDNY 0x0425

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_LIFETIME_EN 0x0426

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_FW_FREE_TAIL 0x0427

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SPEC_SIFS 0x0428
#define REG_RETRY_LIMIT 0x042A
#define REG_TXBF_CTRL 0x042C
#define REG_DARFRC 0x0430

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DARFRCH 0x0434

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RARFRC 0x0438

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RARFRCH 0x043C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RRSR 0x0440

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_RRSR_H 0x0443

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ARFR0 0x0444

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ARFRH0 0x0448

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ARFR1_V1 0x044C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_ARFR1 0x044C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_REG_ARFR_WT0 0x044C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_ARFRH1 0x0450

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_ARFRH1_V1 0x0450

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_REG_ARFR_WT1 0x0450

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_CCK_CHECK 0x0454

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_AMPDU_BURST_CTRL 0x0455

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_BCNQ2_HEAD 0x0455

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_AMPDU_MAX_TIME_V1 0x0455

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_AMPDU_MAX_TIME 0x0456

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BCNQ1_BDNY_V1 0x0456

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TAB_SEL 0x0456

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BCNQ1_BDNY 0x0457

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCN_INVALID_CTRL 0x0457

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8881A_SUPPORT)

#define REG_AMPDU_MAX_LENGTH 0x0458

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_AMPDU_MAX_LENGTH_HT 0x0458

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ACQ_STOP 0x045C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WMAC_LBK_BUF_HD 0x045D

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_NDPA_RATE 0x045D

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TX_HANG_CTRL 0x045E
#define REG_NDPA_OPT_CTRL 0x045F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_FAST_EDCA_CTRL 0x0460

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_AMPDU_MAX_LENGTH_VHT 0x0460

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RD_RESP_PKT_TH 0x0463

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_CMDQ_INFO 0x0464

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_NEW_EDCA_CTRL_V1 0x0464

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_Q4_INFO 0x0468

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACQ_STOP_V2 0x0468

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_Q5_INFO 0x046C
#define REG_Q6_INFO 0x0470
#define REG_Q7_INFO 0x0474

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_WMAC_LBK_BUF_HD_V1 0x0478
#define REG_MGQ_BDNY_V1 0x047A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TXRPT_CTRL 0x047C
#define REG_INIRTS_RATE_SEL 0x0480
#define REG_BASIC_CFEND_RATE 0x0481
#define REG_STBC_CFEND_RATE 0x0482
#define REG_DATA_SC 0x0483

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MACID_SLEEP3 0x0484

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MOREDATA_V1 0x0484

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MACID_SLEEP4 0x0485
#define REG_MACID_SLEEP5 0x0487

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DATA_SC1 0x0487

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MACID_SLEEP1 0x0488

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ARFR2_V1 0x048C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_ARFR2 0x048C
#define REG_ARFRH2 0x0490

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_ARFRH2_V1 0x0490

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ARFR3_V1 0x0494

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_ARFR3 0x0494
#define REG_ARFRH3 0x0498

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_ARFRH3_V1 0x0498

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ARFR4 0x049C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ARFRH4 0x04A0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ARFR5 0x04A4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ARFRH5 0x04A8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TXRPT_START_OFFSET 0x04AC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TRYING_CNT_TH 0x04B0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TRY_CNT_IDX 0x04B0

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_RRSR_CTS 0x04B0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_POWER_STAGE1 0x04B4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_POWER_STAGE2 0x04B8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SW_AMPDU_BURST_MODE_CTRL 0x04BC
#define REG_PKT_LIFE_TIME 0x04C0
#define REG_STBC_SETTING 0x04C4
#define REG_STBC_SETTING2 0x04C5
#define REG_QUEUE_CTRL 0x04C6
#define REG_SINGLE_AMPDU_CTRL 0x04C7
#define REG_PROT_MODE_CTRL 0x04C8
#define REG_BAR_MODE_CTRL 0x04CC
#define REG_RA_TRY_RATE_AGG_LMT 0x04CF

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MACID_SLEEP2 0x04D0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_SLEEP_CTRL 0x04D0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MACID_SLEEP 0x04D4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_SLEEP_INFO 0x04D4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_HW_SEQ0 0x04D8
#define REG_HW_SEQ1 0x04DA
#define REG_HW_SEQ2 0x04DC
#define REG_HW_SEQ3 0x04DE

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_CSI_SEQ 0x04DE

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_NULL_PKT_STATUS 0x04E0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NULL_PKT_STATUS_V1 0x04E0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PTCL_ERR_STATUS 0x04E2

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_PTCL_ERR_STATUS_V1 0x04E2

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PTCL_PKT_NUM 0x04E3

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_NULL_PKT_STATUS_EXTEND 0x04E3

#endif

#if (HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_TRXRPT_MISS_CNT 0x04E3

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HQMGQ_DROP 0x04E4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_VIDEO_ENHANCEMENT_FUN 0x04E4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_NULL_PKT_STATUS_V2 0x04E4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_PRECNT_CTRL 0x04E5

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_NULL_PKT_STATUS_EXTEND_V1 0x04E7

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_BT_POLLUTE_PKTCNT_V1 0x04E8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BT_POLLUTE_PKT_CNT 0x04E8

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_DROP_PKT_NUM 0x04EC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PTCL_DBG 0x04EC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_DROP_NUM 0x04EC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PTCL_DBG_V1 0x04EC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PTCL_TX_RPT 0x04F0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TX_RPT_INFO_L32 0x04F0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TXOP_EXTRA_CTRL 0x04F0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BT_POLLUTE_PKTCNT 0x04F0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TX_RPT_INFO_H32 0x04F4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CPUMGQ_TIMER_CTRL2 0x04F4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PTCL_DBG_OUT 0x04F8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_DUMMY_PAGE4 0x04FC

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DUMMY_PAGE4_V1 0x04FC

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_DUMMY_PAGE4_1 0x04FE

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_MOREDATA 0x04FE

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_EDCA_VO_PARAM 0x0500
#define REG_EDCA_VI_PARAM 0x0504
#define REG_EDCA_BE_PARAM 0x0508
#define REG_EDCA_BK_PARAM 0x050C
#define REG_BCNTCFG 0x0510
#define REG_PIFS 0x0512
#define REG_RDG_PIFS 0x0513
#define REG_SIFS 0x0514

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TSFTR_SYN_OFFSET 0x0518

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FORCE_BCN_IFS_V1 0x0518

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_AGGR_BREAK_TIME 0x051A
#define REG_SLOT 0x051B

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NOA_ON_ERLY_TIME 0x051C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_EDCA_CPUMGQ_PARAM 0x051C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NOA_OFF_ERLY_TIME 0x051D

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_CPUMGQ_PAUSE 0x051E

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PS_TIMER_CTRL 0x051F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TX_PTCL_CTRL 0x0520
#define REG_TXPAUSE 0x0522
#define REG_DIS_TXREQ_CLR 0x0523
#define REG_RD_CTRL 0x0524

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBSSID_CTRL 0x0526

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_P2PPS_CTRL 0x0527

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PKT_LIFETIME_CTRL 0x0528

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_P2PPS_SPEC_STATE 0x052B

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_P2PPS0_SPEC_STATE 0x052B

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PS_TIMER_A_V2 0x052C

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_TBTT_AREA_BLK_4AC 0x052C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_TXOP_LIMIT_CTRL 0x052C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BAR_TX_CTRL 0x0530

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_P2PON_DIS_TXTIME 0x0531

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PS_TIMER_B_V2 0x0534

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_EDCA_REF_CTRL 0x0534

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CCA_TXEN_CNT 0x0534

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8822B_SUPPORT)

#define REG_QUEUE_INCOL_THR 0x0538

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MAX_INTER_COLLISION 0x0538

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8822B_SUPPORT)

#define REG_QUEUE_INCOL_EN 0x053C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MAX_INTER_COLLISION_CNT 0x053C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TBTT_PROHIBIT 0x0540
#define REG_P2PPS_STATE 0x0543

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RD_NAV_NXT 0x0544
#define REG_NAV_PROT_LEN 0x0546

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_FTM_SETTING 0x0548

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_FTM_CTRL 0x0548

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FTM_PTT 0x0548

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT)

#define REG_FTM_TSF_CNT 0x054C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FTM_TSF 0x054C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BCN_CTRL 0x0550

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BCN_CTRL1 0x0551

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BCN_CTRL_CLINT0 0x0551

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBID_NUM 0x0552
#define REG_DUAL_TSF_RST 0x0553
#define REG_MBSSID_BCN_SPACE 0x0554
#define REG_DRVERLYINT 0x0558
#define REG_BCNDMATIM 0x0559

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ATIMWND 0x055A

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_USTIME_TSF 0x055C
#define REG_BCN_MAX_ERR 0x055D
#define REG_RXTSF_OFFSET_CCK 0x055E
#define REG_RXTSF_OFFSET_OFDM 0x055F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TSFTR 0x0560

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFTR0_L 0x0560
#define REG_TSFTR0_H 0x0564

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_TSFTR_1 0x0564

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TSFTR1 0x0568

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFTR1_L 0x0568

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FREERUN_CNT 0x0568

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFTR1_H 0x056C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FREERUN_CNT_1 0x056C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND1 0x0570

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_ATIMWND1_V1 0x0570

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TBTT_PROHIBIT_INFRA 0x0571

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_CTWND 0x0572
#define REG_BCNIVLCUNT 0x0573
#define REG_BCNDROPCTRL 0x0574

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_HGQ_TIMEOUT_PERIOD 0x0575

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TXCMD_TIMEOUT_PERIOD 0x0576
#define REG_MISC_CTRL 0x0577

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFTR2_L 0x0578

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BCN_CTRL_CLINT1 0x0578
#define REG_BCN_CTRL_CLINT2 0x0579
#define REG_BCN_CTRL_CLINT3 0x057A

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_EXTEND_CTRL 0x057B

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFTR2_H 0x057C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_P2PPS1_SPEC_STATE 0x057C
#define REG_P2PPS1_STATE 0x057D
#define REG_P2PPS2_SPEC_STATE 0x057E
#define REG_P2PPS2_STATE 0x057F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_PS_TIMER 0x0580

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER0 0x0580

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TIMER0 0x0584

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER1 0x0584

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TIMER1 0x0588

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER2 0x0588

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TBTT_CTN_AREA 0x058C
#define REG_FORCE_BCN_IFS 0x058E

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_DRVERLYINT_V1 0x058F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TXOP_MIN 0x0590
#define REG_PRE_BKF_TIME 0x0592
#define REG_CROSS_TXOP_CTRL 0x0593

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_FREERUN_CNT_L 0x0594

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TBTT_INT_SHIFT_CLI0 0x0594

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_TBTT_INT_SHIFT_CLI 0x0594

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TBTT_INT_SHIFT_CLI1 0x0595
#define REG_TBTT_INT_SHIFT_CLI2 0x0596
#define REG_TBTT_INT_SHIFT_CLI3 0x0597

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_FREERUN_CNT_H 0x0598

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TBTT_INT_SHIFT_ENABLE 0x0598

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RX_TBTT_SHIFT_V1 0x0598

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND2 0x05A0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ATIMWND_GROUP1 0x05A0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND3 0x05A1

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ATIMWND_GROUP2 0x05A1

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND4 0x05A2

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ATIMWND_GROUP3 0x05A2

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND5 0x05A3

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ATIMWND_GROUP4 0x05A3

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND6 0x05A4

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_DTIM_COUNT_GROUP1 0x05A4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMWND7 0x05A5

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_DTIM_COUNT_GROUP2 0x05A5

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ATIMUGT 0x05A6

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_DTIM_COUNT_GROUP3 0x05A6

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_HIQ_NO_LMT_EN 0x05A7

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_DTIM_COUNT_GROUP4 0x05A7

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_DTIM_COUNTER_ROOT 0x05A8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_HIQ_NO_LMT_EN_V2 0x05A8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_DTIM_COUNTER_VAP1 0x05A9
#define REG_DTIM_COUNTER_VAP2 0x05AA
#define REG_DTIM_COUNTER_VAP3 0x05AB
#define REG_DTIM_COUNTER_VAP4 0x05AC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_MBID_BCNQ_EN 0x05AC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_DTIM_COUNTER_VAP5 0x05AD
#define REG_DTIM_COUNTER_VAP6 0x05AE
#define REG_DTIM_COUNTER_VAP7 0x05AF
#define REG_DIS_ATIM 0x05B0

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_EARLY_128US 0x05B1

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TBTT_HOLD_PREDICT_P1 0x05B2

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_P2PPS1_CTRL 0x05B2

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MULTI_BCN_CS 0x05B3

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_P2PPS2_CTRL 0x05B3

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFT_SHIFT 0x05B4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TIMER0_SRC_SEL 0x05B4
#define REG_NOA_UNIT_SEL 0x05B5

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_P2POFF_DIS_TXTIME 0x05B7

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MBSSID_BCN_SPACE2 0x05B8
#define REG_MBSSID_BCN_SPACE3 0x05BC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_ACMHWCTRL 0x05C0
#define REG_ACMRSTCTRL 0x05C1
#define REG_ACMAVG 0x05C2
#define REG_VO_ADMTIME 0x05C4
#define REG_VI_ADMTIME 0x05C6
#define REG_BE_ADMTIME 0x05C8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MAC_HEADER_NAV_OFFSET 0x05CA
#define REG_DIS_NDPA_NAV_CHECK 0x05CB

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_EDCA_RANDOM_GEN 0x05CC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TXCMD_NOA_SEL 0x05CF

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXCMD_SEL 0x05CF

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_DRVERLYINT2 0x05D0

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_BCNERR_CFG 0x05D0

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_32K_CLK_SEL 0x05D0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_NAN_SETTING 0x05D4

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_BCN_ELY_ADJ 0x05D4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_EARLYINT_ADJUST 0x05D4

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_NAN_BCNSPACE 0x05D8

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_BCNERR_CNT1 0x05D8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BCNERR_CNT 0x05D8

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_NAN_SETTING1 0x05DC

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_BCNERR_CNT2 0x05DC

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BCNERR_CNT_2 0x05DC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_NOA_PARAM 0x05E0

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NOA_PARAM_1 0x05E4
#define REG_NOA_PARAM_2 0x05E8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MU_DBG_INFO 0x05E8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NOA_PARAM_3 0x05EC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MU_DBG_INFO_1 0x05EC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_NOA_SUBIE 0x05ED

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_P2P_RST 0x05F0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_SCH_DBG_SEL 0x05F0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SCHEDULER_RST 0x05F1

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MU_DBG_ERR_FLAG 0x05F2
#define REG_TX_ERR_RECOVERY_RST 0x05F3

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_SCH_DBG 0x05F4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SCH_DBG_VALUE 0x05F4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SCH_TXCMD 0x05F8
#define REG_PAGE5_DUMMY 0x05FC
#define REG_WMAC_CR 0x0600

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_FWPKT_CR 0x0601

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FW_STS_FILTER 0x0602

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_WMAC_BWOPMODE 0x0603

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_BWOPMODE 0x0603

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_TCR 0x0604
#define REG_RCR 0x0608

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RX_PKT_LIMIT 0x060C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RXPKT_LIMIT 0x060C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RX_DLK_TIME 0x060D

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_SDIO_RXINT_LEN_TH 0x1025060E

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RX_DRVINFO_SZ 0x060F
#define REG_MACID 0x0610

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_H 0x0614

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_BSSID 0x0618

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BSSID_H 0x061C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_MAR 0x0620

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MAR_H 0x0624

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBIDCAMCFG_1 0x0628

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MBIDCAMCFG_2 0x062C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MBIDCAM_CFG 0x062C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_WMAC_DEBUG_SEL 0x062C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MCU_TEST_1 0x0630

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_WMAC_TCR_TSFT_OFS 0x0630
#define REG_UDF_THSD 0x0632
#define REG_ZLD_NUM 0x0633

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_MCU_TEST_2 0x0634

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_STMP_THSD 0x0634
#define REG_WMAC_TXTIMEOUT 0x0635

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_MCU_TEST_2_V1 0x0636

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_USTIME_EDCA 0x0638

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ACKTO_CCK 0x0639

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_MAC_SPEC_SIFS 0x063A
#define REG_RESP_SIFS_CCK 0x063C
#define REG_RESP_SIFS_OFDM 0x063E
#define REG_ACKTO 0x0640
#define REG_CTS2TO 0x0641
#define REG_EIFS 0x0642

#endif

#if (HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RPFM_MAP0 0x0644

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_RPFM_MAP1 0x0646

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_RPFM_MAP1_V1 0x0646

#endif

#if (HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RPFM_CAM_CMD 0x0648
#define REG_RPFM_CAM_RWD 0x064C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_NAV_CTRL 0x0650
#define REG_BACAMCMD 0x0654

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BACAMCONTENT 0x0658

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_BACAM_WD 0x0658
#define REG_BACAM_WD_H 0x065C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BACAMCONTENT_H 0x065C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_LBDLY 0x0660

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_LBK_DLY 0x0660
#define REG_BITMAP_CMD 0x0661

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_BACAM_RPMEN 0x0661

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_TX_RX 0x0662

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_BITMAP_CTL 0x0663

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_RXERR_RPT 0x0664
#define REG_WMAC_TRXPTCL_CTL 0x0668

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_TRXPTCL_CTL_H 0x066C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_CAMCMD 0x0670
#define REG_CAMWRITE 0x0674
#define REG_CAMREAD 0x0678
#define REG_CAMDBG 0x067C
#define REG_SECCFG 0x0680

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXFILTER_CATEGORY_1 0x0682
#define REG_RXFILTER_ACTION_1 0x0683
#define REG_RXFILTER_CATEGORY_2 0x0684
#define REG_RXFILTER_ACTION_2 0x0685
#define REG_RXFILTER_CATEGORY_3 0x0686
#define REG_RXFILTER_ACTION_3 0x0687
#define REG_RXFLTMAP3 0x0688
#define REG_RXFLTMAP4 0x068A
#define REG_RXFLTMAP5 0x068C
#define REG_RXFLTMAP6 0x068E

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_WOW_CTRL 0x0690

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NAN_RX_TSF_FILTER 0x0691

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_PS_RX_INFO 0x0692
#define REG_WMMPS_UAPSD_TID 0x0693
#define REG_LPNAV_CTRL 0x0694

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WKFMCAM_NUM 0x0698

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WKFMCAM_CMD 0x0698

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WKFMCAM_RWD 0x069C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RXFLTMAP0 0x06A0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RXFLTER0 0x06A0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RXFLTMAP1 0x06A2

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RXFLTER1 0x06A2

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8881A_SUPPORT)

#define REG_RXFLTMAP 0x06A4

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RXFLTER2 0x06A4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXFLTMAP2 0x06A4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_BCN_PSR_RPT 0x06A8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_FLC_RPC 0x06AC
#define REG_FLC_RPCT 0x06AD
#define REG_FLC_PTS 0x06AE
#define REG_FLC_TRPC 0x06AF

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXPKTMON_CTRL 0x06B0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_STATE_MON 0x06B4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_ERROR_EVT_CTL 0x06B8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ERROR_MON 0x06B8

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RESPINFO 0x06BC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SEARCH_MACID 0x06BC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_COEX_TABLE 0x06C0

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_BT_COEX_TABLE_V1 0x06C0
#define REG_BT_COEX_TABLE2_V1 0x06C4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BT_COEX_TABLE2 0x06C4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_COEX_BREAK_TABLE 0x06C8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_BT_COEX_TABLE_H_V1 0x06CC

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BT_COEX_TABLE_H 0x06CC

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXCMD_0 0x06D0
#define REG_RXCMD_1 0x06D4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WMAC_RESP_TXINFO 0x06D8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_RESP_TXINFO_CFG 0x06D8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BBPSF_CTRL 0x06DC

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_RESP_TXINFO_RATE 0x06DE

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_P2P_RX_BCN_NOA 0x06E0
#define REG_ASSOCIATED_BFMER0_INFO 0x06E4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_SOUNDING_CFG1 0x06E8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_ASSOCIATED_BFMER0_INFO_H 0x06E8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ASSOCIATED_BFMER1_INFO 0x06EC

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_SOUNDING_CFG2 0x06EC
#define REG_SOUNDING_CFG3 0x06F0

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_ASSOCIATED_BFMER1_INFO_H 0x06F0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TX_CSI_RPT_PARAM_BW20 0x06F4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_SOUNDING_CFG0 0x06F4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TX_CSI_RPT_PARAM_BW40 0x06F8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT)

#define REG_ANTCD_INFO 0x06F8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8881A_SUPPORT)

#define REG_TX_CSI_RPT_PARAM_BW80 0x06FC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CSI_PTR 0x06FC

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_CSI_RRSR_V1 0x06FC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_MACID1 0x0700

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID1_1 0x0704

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BSSID1 0x0708

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_CFG_FORCE_LINK_L 0x0709
#define REG_PCIE_CFG_FORCE_LINK_H 0x070A

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BSSID1_1 0x070C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_CFG_DEFAULT_ACK_FREQUENCY 0x070C
#define REG_PCIE_CFG_CX_NFTS 0x070D
#define REG_PCIE_CFG_DEFAULT_ENTR_LATENCY 0x070F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_BCN_PSR_RPT1 0x0710

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_CFG_L1_MISC_SEL 0x0711

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ASSOCIATED_BFMEE_SEL 0x0714

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_ASSOCIATED_BFMEE_SEL_1 0x0714

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_SND_PTCL_CTRL 0x0718

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_CFG_TIMER_CTRL_MAX_FUNC_NUM_OFF 0x0718
#define REG_PCIE_CFG_FORCE_CLKREQ_N_PAD 0x0719
#define REG_PCIE_CFG_TIMER_MODIFIER_FOR_ACK_NAK_LATENCY 0x071A
#define REG_PCIE_CFG_TIMER_MODIFIER_FOR_FLOW_CONTROL_WATCHDOG 0x071B

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_RX_CSI_RPT_INFO 0x071C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_CFG_SKP_INTERVAL_VALUE_L 0x071C
#define REG_PCIE_CFG_SKP_INTERVAL_VALUE_H 0x071D

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RX_CSI_RPT_INFO_H 0x071F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_NS_ARP_CTRL 0x0720
#define REG_NS_ARP_INFO 0x0724

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_CFG_L1_UNIT_SEL 0x0724
#define REG_PCIE_CFG_MIN_CLKREQ_SEL 0x0725

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_NS_ARP_IPADDR 0x0728

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PWR_INT_CTRL 0x0728

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BEAMFORMING_INFO_NSARP_V1 0x0728

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WRITE_RX_CSI_RPT_INFO 0x072C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RX_CSI_RPT_INFO_V1 0x072C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BEAMFORMING_INFO_NSARP 0x072C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_NS_ARP_IPV6_MYADDR 0x0730

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_POWER_MGT_0_V1 0x0730

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_IPV6 0x0730

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_POWER_MGT_1_V1 0x0734

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_IPV6_1 0x0734

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_POWER_MGT_2_V1 0x0738

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_IPV6_2 0x0738

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_POWER_MGT_3_V1 0x073C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_IPV6_3 0x073C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PLCP_HEADER 0x0740
#define REG_TXDRXDMONITOR 0x0744
#define REG_TXDRXDMONITOR_CTL 0x0748

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_RTX_CTX_SUBTYPE_CFG 0x0750

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_WMAC_SWAES_RD0_V1 0x0754

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_SWAES_DIO_B63_B32 0x0754

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_WMAC_SWAES_RD1_V1 0x0758

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_SWAES_DIO_B95_B64 0x0758

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_WMAC_SWAES_RD3_V1 0x075C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_SWAES_DIO_B127_B96 0x075C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_SWAES_CFG 0x0760

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_COEX_V2 0x0762

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_BT_COEX 0x0764

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WLAN_ACT_MSK_CTRL 0x0768

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WLAN_ACT_MASK_CTRL 0x0768

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WLAN_ACT_MASK_CTRL_1 0x076C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_STATISTICS_CTRL 0x076E
#define REG_BT_COEX_ENH_INTF_CTRL 0x076E

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_COEX_ENHANCED_INTR_CTRL 0x076E

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT ||   \
     HALMAC_8881A_SUPPORT)

#define REG_BT_ACT_STATISTICS 0x0770

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_ACT_STATISTICS_1 0x0774

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_STATISTICS_OTH_CTRL 0x0778

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_STATISTICS_CONTROL_REGISTER 0x0778

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_CMD_ID 0x077C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_STATUS_REPORT_REGISTER 0x077C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT__STATUS_RPT 0x077D
#define REG_BT_DATA 0x0780

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_INTERRUPT_CONTROL_REGISTER 0x0780

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WLAN_RPT_ 0x0781
#define REG_BT_ISR_CTRL 0x0783
#define REG_WLAN_RPT_TO_CTR 0x0784

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WLAN_REPORT_TIME_OUT_CONTROL_REGISTER 0x0784

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_ISOLATION_TABLE 0x0785

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_ISOLATION_TABLE_REGISTER_REGISTER 0x0785

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_ISOLATION_TABLE_REGISTER_REGISTER_1 0x0788
#define REG_BT_ISOLATION_TABLE_REGISTER_REGISTER_2 0x078C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_ISR_STA 0x078F

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_INTERRUPT_STATUS_REGISTER 0x078F

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_TDMA_TIME_AND_RPT_SAM_SET 0x0790

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_TDMA_TIME_REGISTER 0x0790

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_CH_INFO 0x0794

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BT_ACT_REGISTER 0x0794

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_BT_STATIC_INFO_EXT 0x0795
#define REG_LTR_IDLE_LATENCY 0x0798

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_LTR_IDLE_LATENCY_V2 0x0798

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_OBFF_CTRL_BASIC 0x0798

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_LTR_ACTIVE_LATENCY 0x079C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_LTR_ACTIVE_LATENCY_V2 0x079C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_OBFF_CTRL2_TIMER 0x079C

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_OBFF_CTRL 0x07A0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_LTR_CTRL_BASIC 0x07A0

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_LTR_CTRL 0x07A4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_LTR_CTRL2_TIMER_THRESHOLD 0x07A4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_LTR_CTRL2 0x07A8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_LTR_IDLE_LATENCY_V1 0x07A8
#define REG_LTR_ACTIVE_LATENCY_V1 0x07AC

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_ANTTRN_CTRL 0x07B0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_ANTTRN_CTR_V1 0x07B0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANTENNA_TRAINING_CONTROL_REGISTER 0x07B0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_SMART_ANT_CONDITION 0x07B0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_ANTTRN_CTR 0x07B4

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_ANTENNA_TRAINING_CONTROL_REGISTER_1 0x07B4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_SMART_ANT_CTRL 0x07B4

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WMAC_PKTCNT_RWD 0x07B8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_CONTROL_FRAME_REPORT 0x07B8

#endif

#if (HALMAC_8192E_SUPPORT || HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT ||   \
     HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT || HALMAC_8881A_SUPPORT)

#define REG_WMAC_PKTCNT_CTRL 0x07BC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_CONTROL_FRAME_CNT_CTRL 0x07BC

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_WL2LTECOEX_INDIRECT_ACCESS_CTRL 0x07C0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_IQ_DUMP 0x07C0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_WL2LTECOEX_INDIRECT_ACCESS_WRITE_DATA 0x07C4

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_IQ_DUMP_1 0x07C4

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_WL2LTECOEX_INDIRECT_ACCESS_READ_DATA 0x07C8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_IQ_DUMP_2 0x07C8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_FTM_CTL 0x07CC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_IQ_MDPK_FUNC 0x07CE

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_IQ_DUMP_EXT 0x07CF

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT)

#define REG_OFDM_CCK_LEN_MASK 0x07D0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_OPTION_FUNCTION 0x07D0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_FA_FILTER1 0x07D4

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_OPTION_FUNCTION_1 0x07D4

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_FA_FILTER2 0x07D8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_OPTION_FUNCTION_2 0x07D8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RX_FILTER_FUNCTION 0x07DA

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_NAN_FUN 0x07DC
#define REG_NAN_CTL 0x07E0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_NDP_SIG 0x07E0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RX_NAN_ADDR_FILTER 0x07E4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_TXCMD_INFO_FOR_RSP_PKT 0x07E4

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_NAN_ADDR 0x07E8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TXCMD_INFO_FOR_RSP_PKT_1 0x07E8

#endif

#if (HALMAC_8814AMP_SUPPORT)

#define REG_SEC_OPT 0x07E8

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_RXA1_MASK 0x07EC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_SEC_OPT_V2 0x07EC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WSEC_OPTION 0x07EC

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_DUMP_FUNC 0x07F0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RTS_ADDRESS_0 0x07F0

#endif

#if (HALMAC_8814AMP_SUPPORT)

#define REG_RTS_ADDR0 0x07F0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MASK_LA_MAC 0x07F4

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RTS_ADDRESS_0_1 0x07F4

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_MATCH_REF_MAC 0x07F8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RTS_ADDRESS_1 0x07F8

#endif

#if (HALMAC_8814AMP_SUPPORT)

#define REG_RTS_ADDR1 0x07F8

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_LA_DUMP_FUNC_EXT 0x07FC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RTS_ADDRESS_1_1 0x07FC

#endif

#if (HALMAC_8822B_SUPPORT)

#define REG__RPFM_MAP1 0x07FE

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SYS_CFG3 0x1000

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPARSW_MAC_0 0x1010
#define REG_ANAPARSW_MAC_1 0x1014
#define REG_ANAPAR_MAC_0 0x1018
#define REG_ANAPAR_MAC_1 0x101C
#define REG_ANAPAR_MAC_2 0x1020

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ANAPAR_MAC_3 0x1024
#define REG_ANAPAR_MAC_4 0x1028
#define REG_ANAPAR_MAC_5 0x102C
#define REG_ANAPAR_MAC_6 0x1030

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_SYS_CFG4 0x1034

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ANAPAR_MAC_7 0x1034
#define REG_ANAPAR_MAC_8 0x1038

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPAR_XTAL_0 0x1040
#define REG_ANAPAR_XTAL_1 0x1044
#define REG_ANAPAR_XTAL_2 0x1048

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPAR_XTAL_3 0x104C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ANAPAR_XTAL_AAC 0x104C
#define REG_ANAPAR_XTAL_R_ONLY 0x1050

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPAR_XTAL_AACK_0 0x1054

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_CPHY_LDO 0x1054

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPAR_XTAL_AACK_1 0x1058

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_CPHY_BG 0x1058
#define REG_HIMR_4 0x1060

#endif

#if (HALMAC_8822C_SUPPORT)

#define REG_XTAL_AAC_OUTPUT 0x1060

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_ANAPAR_XTAL_MODE_DECODER 0x1064

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HISR_4 0x1064
#define REG_HIMR_5 0x1068
#define REG_HISR_5 0x106C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SYS_CFG5 0x1070

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_REGU_32K_1 0x1078

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HIMR_6 0x1078

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_REGU_32K_2 0x107C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HISR_6 0x107C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CPU_DMEM_CON 0x1080

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BOOT_REASON 0x1088

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HIMR4 0x1090

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DATA_CPU_CTL0 0x1090

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HISR4 0x1094

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_DATA_CPU_CTL1 0x1094

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HIMR5 0x1098

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXDMA_STOP_HIMR 0x1098

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HISR5 0x109C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXDMA_STOP_HISR 0x109C
#define REG_TXDMA_START_HIMR 0x10A0
#define REG_TXDMA_START_HISR 0x10A4

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_NFC_PAD_CTRL 0x10A8

#endif

#if (HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_NFCPAD_CTRL 0x10A8

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HIMR2 0x10B0
#define REG_HISR2 0x10B4
#define REG_HIMR3 0x10B8
#define REG_HISR3 0x10BC
#define REG_SW_MDIO 0x10C0

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_SW_FLUSH 0x10C4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_DBG_GPIO_BMUX 0x10C8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HIMR_7 0x10C8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_FPGA_TAG 0x10CC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HISR_7 0x10CC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_WL_DSS_CTRL0 0x10D0

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_H2C_PKT_READADDR 0x10D0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_WL_DSS_STATUS0 0x10D4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_H2C_PKT_WRITEADDR 0x10D4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_WL_DSS_CTRL1 0x10D8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MEM_PWR_CRTL 0x10D8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_WL_DSS_STATUS1 0x10DC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FW_DRV_HANDSHAKE 0x10DC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_FW_DBG0 0x10E0
#define REG_FW_DBG1 0x10E4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_FW_DBG2 0x10E8
#define REG_FW_DBG3 0x10EC
#define REG_FW_DBG4 0x10F0
#define REG_FW_DBG5 0x10F4

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FW_DBG6 0x10F8
#define REG_FW_DBG7 0x10FC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_CR_EXT 0x1100

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TC9_CTRL 0x1104
#define REG_TC10_CTRL 0x1108
#define REG_TC11_CTRL 0x110C
#define REG_TC12_CTRL 0x1110

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FWFF 0x1114

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_RXFF_PTR_V1 0x1118
#define REG_RXFF_WTR_V1 0x111C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FE2IMR 0x1120
#define REG_FE2ISR 0x1124
#define REG_FE3IMR 0x1128
#define REG_FE3ISR 0x112C
#define REG_FE4IMR 0x1130
#define REG_FE4ISR 0x1134
#define REG_FT1IMR 0x1138
#define REG_FT1ISR 0x113C
#define REG_SPWR0 0x1140
#define REG_SPWR1 0x1144
#define REG_SPWR2 0x1148
#define REG_SPWR3 0x114C
#define REG_POWSEQ 0x1150

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TC7_CTRL_V1 0x1158
#define REG_TC8_CTRL_V1 0x115C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_RXBCN_TBTT_INTERVAL_PORT0TO3 0x1160

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_RX_BCN_TBTT_ITVL0 0x1160

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_RXBCN_TBTT_INTERVAL_PORT4 0x1164

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_RX_BCN_TBTT_ITVL1 0x1164

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_FWIMR1 0x1168
#define REG_FWISR1 0x116C
#define REG_FWIMR2 0x1170

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_IO_WRAP_ERR_FLAG 0x1170

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_FWISR2 0x1174

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWIMR3 0x1178
#define REG_FWISR3 0x117C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_SPEED_SENSOR 0x1180
#define REG_SPEED_SENSOR1 0x1184
#define REG_SPEED_SENSOR2 0x1188
#define REG_SPEED_SENSOR3 0x118C
#define REG_SPEED_SENSOR4 0x1190
#define REG_SPEED_SENSOR5 0x1194

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RXPKTBUF_1_MAX_ADDR 0x1198
#define REG_RXFWBUF_1_MAX_ADDR 0x119C
#define REG_IO_WRAP_ERR_FLAG_V1 0x11A0
#define REG_RXPKTBUF_1_READ 0x11A4
#define REG_RXPKTBUF_1_WRITE 0x11A8
#define REG_BUFF_DBGUG 0x11AC
#define REG_RFE_CTRL_PAD_E2 0x11B0
#define REG_RFE_CTRL_PAD_SR 0x11B4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT)

#define REG_EXT_QUEUE_REG 0x11C0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_H2C_PRIORITY_SEL 0x11C0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT)

#define REG_COUNTER_CONTROL 0x11C4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_COUNTER_CTRL 0x11C4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT)

#define REG_COUNTER_TH 0x11C8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_COUNTER_THRESHOLD 0x11C8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_COUNTER_SET 0x11CC
#define REG_COUNTER_OVERFLOW 0x11D0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT)

#define REG_TDE_LEN_TH 0x11D4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_TXDMA_LEN_THRESHOLD 0x11D4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT)

#define REG_RDE_LEN_TH 0x11D8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_RXDMA_LEN_THRESHOLD 0x11D8

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT)

#define REG_PCIE_EXEC_TIME 0x11DC

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_PCIE_EXEC_TIME_THRESHOLD 0x11DC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_FT2IMR 0x11E0
#define REG_FT2ISR 0x11E4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_MSG2 0x11F0
#define REG_MSG3 0x11F4
#define REG_MSG4 0x11F8
#define REG_MSG5 0x11FC
#define REG_DDMA_CH0SA 0x1200
#define REG_DDMA_CH0DA 0x1204
#define REG_DDMA_CH0CTRL 0x1208
#define REG_DDMA_CH1SA 0x1210
#define REG_DDMA_CH1DA 0x1214
#define REG_DDMA_CH1CTRL 0x1218
#define REG_DDMA_CH2SA 0x1220
#define REG_DDMA_CH2DA 0x1224
#define REG_DDMA_CH2CTRL 0x1228
#define REG_DDMA_CH3SA 0x1230
#define REG_DDMA_CH3DA 0x1234
#define REG_DDMA_CH3CTRL 0x1238
#define REG_DDMA_CH4SA 0x1240
#define REG_DDMA_CH4DA 0x1244
#define REG_DDMA_CH4CTRL 0x1248
#define REG_DDMA_CH5SA 0x1250
#define REG_DDMA_CH5DA 0x1254

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_REG_DDMA_CH5CTRL 0x1258

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DDMA_CH5CTRL 0x1258

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_DDMA_INT_MSK 0x12E0
#define REG_DDMA_CHSTATUS 0x12E8
#define REG_DDMA_CHKSUM 0x12F0
#define REG_DDMA_MONITOR 0x12FC

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_STC_INT_CS 0x1300
#define REG_ST_INT_CFG 0x1304

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH4_ACH5_TXBD_NUM 0x130C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_CMU_DLY_CTRL 0x1310
#define REG_CMU_DLY_CFG 0x1314

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWCMDQ_TXBD_IDX 0x1318
#define REG_P0HI8Q_TXBD_IDX 0x131C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_H2CQ_TXBD_DESA 0x1320

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_H2CQ_TXBD_DESA_L 0x1320
#define REG_H2CQ_TXBD_DESA_H 0x1324

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_H2CQ_TXBD_NUM 0x1328

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_H2CQ_TXBD_IDX 0x132C

#endif

#if (HALMAC_8192F_SUPPORT || HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT ||   \
     HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_H2CQ_CSR 0x1330

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI9Q_TXBD_IDX 0x1334
#define REG_P0HI10Q_TXBD_IDX 0x1338
#define REG_P0HI11Q_TXBD_IDX 0x133C
#define REG_P0HI12Q_TXBD_IDX 0x1340

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_CPL_BUFFER_MONITOR 0x1344

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI13Q_TXBD_IDX 0x1344

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PTM_LOCAL_CLOCK 0x1348

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI14Q_TXBD_IDX 0x1348

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PTM_LOCAL_CLOCK_H 0x134C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_P0HI15Q_TXBD_IDX 0x134C

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_TSFT_PTM_DIFF 0x1350

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_AXI_EXCEPT_CS 0x1350

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_CHANGE_PCIE_SPEED 0x1350

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_PTM_CTRL_STATUS 0x1354

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_AXI_EXCEPT_TIME 0x1354

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DEBUG_STATE1 0x1354

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_QUEUE_HEADER_CUR_REMAIN 0x1358

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI8Q_TXBD_IDX 0x1358

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DEBUG_STATE2 0x1358

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_QUEUE_HEADER_MIN_REMAIN 0x135C

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI9Q_TXBD_IDX 0x135C

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_DEBUG_STATE3 0x135C

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI10Q_TXBD_IDX 0x1360

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH5_TXBD_DESA_L 0x1360

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI11Q_TXBD_IDX 0x1364

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH5_TXBD_DESA_H 0x1364

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI12Q_TXBD_IDX 0x1368

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH6_TXBD_DESA_L 0x1368

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI13Q_TXBD_IDX 0x136C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH6_TXBD_DESA_H 0x136C

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI14Q_TXBD_IDX 0x1370

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH7_TXBD_DESA_L 0x1370

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI15Q_TXBD_IDX 0x1374

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH7_TXBD_DESA_H 0x1374

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI8Q_TXBD_DESA 0x1378

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH8_TXBD_DESA_L 0x1378

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CHNL_DMA_CFG_V1 0x137C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH8_TXBD_DESA_H 0x137C

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI9Q_TXBD_DESA 0x1380

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH9_TXBD_DESA_L 0x1380
#define REG_ACH9_TXBD_DESA_H 0x1384

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI10Q_TXBD_DESA 0x1388

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH10_TXBD_DESA_L 0x1388
#define REG_ACH10_TXBD_DESA_H 0x138C

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI11Q_TXBD_DESA 0x1390

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH11_TXBD_DESA_L 0x1390
#define REG_ACH11_TXBD_DESA_H 0x1394

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI12Q_TXBD_DESA 0x1398

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH12_TXBD_DESA_L 0x1398
#define REG_ACH12_TXBD_DESA_H 0x139C

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI13Q_TXBD_DESA 0x13A0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH13_TXBD_DESA_L 0x13A0
#define REG_ACH13_TXBD_DESA_H 0x13A4

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI14Q_TXBD_DESA 0x13A8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI0Q_TXBD_DESA_L 0x13A8
#define REG_HI0Q_TXBD_DESA_H 0x13AC

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI15Q_TXBD_DESA 0x13B0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI1Q_TXBD_DESA_L 0x13B0

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_HISR0_V1 0x13B4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI1Q_TXBD_DESA_H 0x13B4

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI8Q_TXBD_NUM 0x13B8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI2Q_TXBD_DESA_L 0x13B8

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI9Q_TXBD_NUM 0x13BA
#define REG_HI10Q_TXBD_NUM 0x13BC

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_HISR1_V1 0x13BC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI2Q_TXBD_DESA_H 0x13BC

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI11Q_TXBD_NUM 0x13BE
#define REG_HI12Q_TXBD_NUM 0x13C0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI3Q_TXBD_DESA_L 0x13C0

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI13Q_TXBD_NUM 0x13C2
#define REG_HI14Q_TXBD_NUM 0x13C4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI3Q_TXBD_DESA_H 0x13C4

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_HI15Q_TXBD_NUM 0x13C6
#define REG_HIQ_DMA_STOP 0x13C8

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI4Q_TXBD_DESA_L 0x13C8
#define REG_HI4Q_TXBD_DESA_H 0x13CC
#define REG_HI5Q_TXBD_DESA_L 0x13D0
#define REG_HI5Q_TXBD_DESA_H 0x13D4
#define REG_HI6Q_TXBD_DESA_L 0x13D8
#define REG_HI6Q_TXBD_DESA_H 0x13DC
#define REG_HI7Q_TXBD_DESA_L 0x13E0
#define REG_HI7Q_TXBD_DESA_H 0x13E4
#define REG_ACH8_ACH9_TXBD_NUM 0x13E8
#define REG_ACH10_ACH11_TXBD_NUM 0x13EC
#define REG_ACH12_ACH13_TXBD_NUM 0x13F0

#endif

#if (HALMAC_8814B_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_OLD_DEHANG 0x13F4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_ACH4_TXBD_DESA_L 0x13F8
#define REG_ACH4_TXBD_DESA_H 0x13FC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_Q0_Q1_INFO 0x1400

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ARFR6 0x1400

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MU_OFFSET 0x1400

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_Q2_Q3_INFO 0x1404

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_ARFRH6 0x1404

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_Q4_Q5_INFO 0x1408

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ARFR7 0x1408

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_Q6_Q7_INFO 0x140C

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_ARFRH7 0x140C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_MGQ_HIQ_INFO 0x1410

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_ARFR8 0x1410

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT ||   \
     HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || \
     HALMAC_8822C_SUPPORT)

#define REG_CMDQ_BCNQ_INFO 0x1414

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_ARFRH8 0x1414

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_USEREG_SETTING 0x1420

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_LOOPBACK_OPTION 0x1420

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_AESIV_SETTING 0x1424

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BF0_TIME_SETTING 0x1428
#define REG_BF1_TIME_SETTING 0x142C
#define REG_BF_TIMEOUT_EN 0x1430

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_RELEASE0 0x1434

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_RELEASE_INFO 0x1434

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_RELEASE1 0x1438

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_RELEASE_SUCCESS_INFO 0x1438

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_RELEASE2 0x143C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_RELEASE_CTRL 0x143C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_RELEASE3 0x1440
#define REG_MACID_RELEASE_SETTING 0x1444

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_FAST_EDCA_VOVI_SETTING 0x1448
#define REG_FAST_EDCA_BEBK_SETTING 0x144C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_DROP0 0x1450

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_DROP_INFO 0x1450

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_DROP1 0x1454

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MACID_DROP_CTRL 0x1454

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID_DROP2 0x1458
#define REG_MACID_DROP3 0x145C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_R_MACID_RELEASE_SUCCESS_0 0x1460
#define REG_R_MACID_RELEASE_SUCCESS_1 0x1464
#define REG_R_MACID_RELEASE_SUCCESS_2 0x1468
#define REG_R_MACID_RELEASE_SUCCESS_3 0x146C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_MGG_FIFO_CRTL 0x1470

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MGQ_FIFO_WRITE_POINTER 0x1470
#define REG_MGQ_FIFO_READ_POINTER 0x1472
#define REG_MGQ_FIFO_ENABLE 0x1472

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_MGG_FIFO_INT 0x1474

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MGQ_FIFO_RELEASE_INT_MASK 0x1474
#define REG_MGQ_FIFO_RELEASE_INT_FLAG 0x1476

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_MGG_FIFO_LIFETIME 0x1478

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MGQ_FIFO_VALID_MAP 0x1478
#define REG_MGQ_FIFO_LIFETIME 0x147A

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_R_MACID_RELEASE_SUCCESS_CLEAR_OFFSET 0x147C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8821C_SUPPORT || \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SHCUT_SETTING 0x1480

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PKT_TRANS 0x1480

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814AMP_SUPPORT || HALMAC_8814B_SUPPORT || \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SHCUT_LLC_ETH_TYPE0 0x1484
#define REG_SHCUT_LLC_ETH_TYPE1 0x1488
#define REG_SHCUT_LLC_OUI0 0x148C
#define REG_SHCUT_LLC_OUI1 0x1490
#define REG_SHCUT_LLC_OUI2 0x1494

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8822B_SUPPORT)

#define REG_SHCUT_LLC_OUI3 0x1498

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWCMDQ_CTRL 0x14A0
#define REG_FWCMDQ_PAGE 0x14A4
#define REG_FWCMDQ_INFO 0x14A8
#define REG_FWCMDQ_HOLD_PKTNUM 0x14AC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MU_TX_CTL 0x14C0

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_MU_TX_CTRL 0x14C0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MU_STA_GID_VLD 0x14C4
#define REG_MU_STA_USER_POS_INFO 0x14C8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MU_STA_USER_POS_INFO_H 0x14CC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CHNL_INFO_CTRL 0x14D0

#endif

#if (HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_MU_TRX_DBG_CNT 0x14D0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CHNL_IDLE_TIME 0x14D4
#define REG_CHNL_BUSY_TIME 0x14D8

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MU_TRX_DBG_CNT_V1 0x14DC

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_NEW_EDCA_CTRL 0x14F0

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_SU_DURATION 0x14F0
#define REG_MU_DURATION 0x14F2

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_SWPS_CTRL 0x14F4

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_HW_NDPA_RTY_LIMIT 0x14F4

#endif

#if (HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_SWPS_PKT_TH 0x14F6

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8814B_SUPPORT)

#define REG_SWPS_TIME_TH 0x14F8
#define REG_MACID_SWPS_EN 0x14FC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_CPUMGQ_TX_TIMER 0x1500

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PORT_CTRL_SEL 0x1500
#define REG_PORT_CTRL_CFG 0x1501

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER_A 0x1504

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TBTT_PROHIBIT_CFG 0x1504
#define REG_DRVERLYINT_CFG 0x1507

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER_B 0x1508

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCNDMATIM_CFG 0x1508
#define REG_CTWND_CFG 0x1509
#define REG_BCNIVLCUNT_CFG 0x150A
#define REG_EARLY_128US_CFG 0x150B

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER_C 0x150C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TSFTR_SYNC_OFFSET_CFG 0x150C
#define REG_TSFTR_SYNC_CTRL_CFG 0x150F

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_PS_TIMER_ABC_CPUMGQ_TIMER_CRTL 0x1510

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCN_SPACE_CFG 0x1510
#define REG_EARLY_INT_ADJUST_CFG 0x1512

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_CPUMGQ_TX_TIMER_EARLY 0x1514
#define REG_PS_TIMER_A_EARLY 0x1515
#define REG_PS_TIMER_B_EARLY 0x1516
#define REG_PS_TIMER_C_EARLY 0x1517

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8812F_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_CPUMGQ_PARAMETER 0x1518

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_SW_TBTT_TSF_INFO 0x151C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TSF_SYN_CTRL0 0x1520

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TSF_SYNC_ADJ 0x1520

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TSFTR_LOW 0x1520

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_TSF_SYN_CTRL1 0x1521
#define REG_TSF_SYN_OFFSET0 0x1522

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TSF_SYN_OFFSET1 0x1524

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TSF_ADJ_VLAUE 0x1524

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TSFTR_HIGH 0x1524

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT)

#define REG_TSF_SYN_OFFSET2 0x1528

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TSF_ADJ_VLAUE_2 0x1528

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BCN_ERR_CNT_MAC 0x1528
#define REG_BCN_ERR_CNT_EDCCA 0x1529
#define REG_BCN_ERR_CNT_CCA 0x152A
#define REG_BCN_ERR_CNT_INVALID 0x152B
#define REG_BCN_ERR_CNT_OTHERS 0x152C
#define REG_RX_BCN_TIMER 0x152D

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_TSF_SYN_COMPARE_VALUE_L 0x1530

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_TSF_SYN_COMPARE_VALUE 0x1530

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TBTT_CTN_AREA_V1 0x1530
#define REG_BCN_MAX_ERR_V1 0x1531
#define REG_RXTSF_OFFSET_CCK_V1 0x1532
#define REG_RXTSF_OFFSET_OFDM_V1 0x1533

#endif

#if (HALMAC_8197G_SUPPORT)

#define REG_TSF_SYN_COMPARE_VALUE_H 0x1534

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_SUB_BCN_SPACE 0x1534
#define REG_MBID_NUM_V1 0x1535
#define REG_MBSSID_CTRL_V1 0x1536
#define REG_USTIME_TSF_V1 0x1538
#define REG_BW_CFG 0x1539
#define REG_ATIMWND_CFG 0x153A
#define REG_DTIM_COUNTER_CFG 0x153B
#define REG_ATIM_DTIM_CTRL_SEL 0x153C
#define REG_ATIMUGT_V1 0x153D
#define REG_BCNDROPCTRL_V1 0x153E
#define REG_DIS_ATIM_V1 0x1540
#define REG_HIQ_NO_LMT_EN_V1 0x1544
#define REG_P2PPS_CTRL_V1 0x1548
#define REG_P2PPS_SPEC_STATE_V1 0x154A
#define REG_P2PPS_STATE_V1 0x154B
#define REG_P2PPS1_CTRL_V1 0x154C
#define REG_P2PPS1_SPEC_STATE_V1 0x154E
#define REG_P2PPS1_STATE_V1 0x154F
#define REG_P2PPS2_CTRL_V1 0x1550
#define REG_P2PPS2_SPEC_STATE_V1 0x1552
#define REG_P2PPS2_STATE_V1 0x1553
#define REG_P2PON_DIS_TXTIME_V1 0x1554
#define REG_P2POFF_DIS_TXTIME_V1 0x1555
#define REG_CHG_POWER_BCN_AREA 0x1556
#define REG_NOA_SEL 0x1557
#define REG_NOA_PARAM_V1 0x1558
#define REG_NOA_PARAM_1_V1 0x155C
#define REG_NOA_PARAM_2_V1 0x1560
#define REG_NOA_PARAM_3_V1 0x1564
#define REG_NOA_ON_ERLY_TIME_V1 0x1568
#define REG_NOA_OFF_ERLY_TIME_V1 0x1569

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_P2PPS_HW_AUTO_PAUSE_CTRL 0x156C
#define REG_P2PPS1_HW_AUTO_PAUSE_CTRL 0x1570
#define REG_P2PPS2_HW_AUTO_PAUSE_CTRL 0x1574

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_RX_TBTT_SHIFT 0x1578
#define REG_FREERUN_CNT_LOW 0x1580
#define REG_FREERUN_CNT_HIGH 0x1584
#define REG_CPUMGQ_TX_TIMER_V1 0x1588
#define REG_PS_TIMER_0 0x158C
#define REG_PS_TIMER_1 0x1590
#define REG_PS_TIMER_2 0x1594
#define REG_PS_TIMER_3 0x1598
#define REG_PS_TIMER_4 0x159C
#define REG_PS_TIMER_5 0x15A0
#define REG_PS_TIMER_01_CTRL 0x15A4
#define REG_PS_TIMER_23_CTRL 0x15A8
#define REG_PS_TIMER_45_CTRL 0x15AC
#define REG_CPUMGQ_FREERUN_TIMER_CTRL 0x15B0
#define REG_CPUMGQ_PROHIBIT 0x15B4
#define REG_TIMER_COMPARE 0x15C0
#define REG_TIMER_COMPARE_VALUE_LOW 0x15C4
#define REG_TIMER_COMPARE_VALUE_HIGH 0x15C8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_SCHEDULER_COUNTER 0x15D0

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BCN_PSR_RPT2 0x1600
#define REG_BCN_PSR_RPT3 0x1604
#define REG_BCN_PSR_RPT4 0x1608
#define REG_A1_ADDR_MASK 0x160C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_RXPSF_CTRL 0x1610
#define REG_RXPSF_TYPE_CTRL 0x1614
#define REG_CAM_ACCESS_CTRL 0x1618

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_HT_SND_REF_RATE 0x161C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_CUT_AMSDU_CTRL 0x161C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_MACID2 0x1620

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID2_H 0x1624

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BSSID2 0x1628

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BSSID2_H 0x162C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_MACID3 0x1630

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID3_H 0x1634

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BSSID3 0x1638

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BSSID3_H 0x163C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_MACID4 0x1640

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MACID4_H 0x1644

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814A_SUPPORT || HALMAC_8814AMP_SUPPORT || \
     HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_BSSID4 0x1648

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_BSSID4_H 0x164C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NOA_REPORT 0x1650

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_NOA_REPORT_1 0x1654
#define REG_NOA_REPORT_2 0x1658
#define REG_NOA_REPORT_3 0x165C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PWRBIT_SETTING 0x1660

#endif

#if (HALMAC_8822B_SUPPORT)

#define REG_MACRX_HANG_TIMER_CONTROL 0x1661

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_GENERAL_OPTION 0x1664

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT)

#define REG_RXAI_CTRL 0x1668

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_FWPHYFF_RCR 0x1668
#define REG_ADDRCAM_WRITE_CONTENT 0x166C
#define REG_ADDRCAM_READ_CONTENT 0x1670
#define REG_ADDRCAM_CFG 0x1674

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_CSI_RRSR 0x1678

#endif

#if (HALMAC_8198F_SUPPORT)

#define REG_WMAC_CSI_FRAME_RRSR_SETTING 0x1678

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8822B_SUPPORT)

#define REG_WMAC_MU_BF_OPTION 0x167C

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_MU_BF_OPTION 0x167C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822C_SUPPORT)

#define REG_WMAC_PAUSE_BB_CLR_TH 0x167D

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8198F_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT)

#define REG_WMAC_MU_ARB 0x167E

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG__WMAC_MULBK_BUF 0x167E

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_WMAC_MULBK_BUF 0x167E

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_MU_OPTION 0x167F
#define REG_WMAC_MU_BF_CTL 0x1680
#define REG_WMAC_MU_BFRPT_PARA 0x1682
#define REG_WMAC_ASSOCIATED_MU_BFMEE2 0x1684
#define REG_WMAC_ASSOCIATED_MU_BFMEE3 0x1686
#define REG_WMAC_ASSOCIATED_MU_BFMEE4 0x1688
#define REG_WMAC_ASSOCIATED_MU_BFMEE5 0x168A
#define REG_WMAC_ASSOCIATED_MU_BFMEE6 0x168C
#define REG_WMAC_ASSOCIATED_MU_BFMEE7 0x168E

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_BB_STOP_RX_COUNTER 0x1690
#define REG_WMAC_PLCP_MONITOR 0x1694

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_PLCP_MONITOR_MUTX 0x1698

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_WMAC_DEBUG_PORT 0x1698

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WMAC_CSIDMA_CFG 0x169C

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_0 0x16A0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_0_H 0x16A4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_1 0x16A8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_1_H 0x16AC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_2 0x16B0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_2_H 0x16B4

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_3 0x16B8

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_3_H 0x16BC

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_4 0x16C0

#endif

#if (HALMAC_8197G_SUPPORT || HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT ||   \
     HALMAC_8821C_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_TRANSMIT_ADDRSS_4_H 0x16C4

#endif

#if (HALMAC_8812F_SUPPORT)

#define REG_SND_AID12 0x16D0
#define REG_SND_PKT_INFO 0x16D2

#endif

#if (HALMAC_8197F_SUPPORT || HALMAC_8197G_SUPPORT || HALMAC_8198F_SUPPORT ||   \
     HALMAC_8812F_SUPPORT || HALMAC_8814B_SUPPORT || HALMAC_8821C_SUPPORT ||   \
     HALMAC_8822B_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_WL2LTECOEX_INDIRECT_ACCESS_CTRL_V1 0x1700
#define REG_WL2LTECOEX_INDIRECT_ACCESS_WRITE_DATA_V1 0x1704
#define REG_WL2LTECOEX_INDIRECT_ACCESS_READ_DATA_V1 0x1708

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_BIST_RSTN0 0x2100
#define REG_BIST_RSTN2 0x2108
#define REG_BIST_MODE_NRML0 0x2110
#define REG_BIST_MODE_NRML1 0x2114
#define REG_BIST_MODE_NRML2 0x2118
#define REG_BIST_MODE_NRML3 0x211C
#define REG_BIST_DONE_NRML_MAC 0x2150
#define REG_BIST_DONE_NRML1 0x2158
#define REG_BIST_DONE_DRF_MAC 0x2160
#define REG_BIST_DONE_DRF 0x2164
#define REG_BIST_DONE_DRF1 0x2168
#define REG_BIST_FAIL_NRML_MAC 0x2170
#define REG_BIST_FAIL_NRML 0x2174
#define REG_BIST_FAIL_NRML1 0x2178
#define REG_BIST_FAIL_NRML_MAC_V1 0x2180
#define REG_BIST_FAIL_NRML_V1 0x2184
#define REG_BIST_FAIL_NRML1_V1 0x2188
#define REG_BIST_MISR_DATAOUT 0x2190
#define REG_BIST_MISR_DATAOUT1 0x2194
#define REG_BIST_MISR_DATAOUT_CPU 0x2198
#define REG_BIST_MISR_DATAOUT_CPU1 0x219C
#define REG_BIST_MISR_DATAOUT_CPU2 0x21A0
#define REG_BIST_MISR_DATOUT_CPU3 0x21A4
#define REG_DMA_RQPN_INFO_0 0x2200
#define REG_DMA_RQPN_INFO_1 0x2204
#define REG_DMA_RQPN_INFO_2 0x2208
#define REG_DMA_RQPN_INFO_3 0x220C
#define REG_DMA_RQPN_INFO_4 0x2210
#define REG_DMA_RQPN_INFO_5 0x2214
#define REG_DMA_RQPN_INFO_6 0x2218
#define REG_DMA_RQPN_INFO_7 0x221C
#define REG_DMA_RQPN_INFO_8 0x2220
#define REG_DMA_RQPN_INFO_9 0x2224
#define REG_DMA_RQPN_INFO_10 0x2228
#define REG_DMA_RQPN_INFO_11 0x222C
#define REG_DMA_RQPN_INFO_12 0x2230
#define REG_DMA_RQPN_INFO_13 0x2234
#define REG_DMA_RQPN_INFO_14 0x2238
#define REG_DMA_RQPN_INFO_15 0x223C
#define REG_DMA_RQPN_INFO_16 0x2240
#define REG_HWAMSDU_CTL1 0x2250
#define REG_HWAMSDU_CTL2 0x2254
#define REG_HI8Q_TXBD_DESA_L 0x2300
#define REG_HI8Q_TXBD_DESA_H 0x2304
#define REG_HI9Q_TXBD_DESA_L 0x2308
#define REG_HI9Q_TXBD_DESA_H 0x230C
#define REG_HI10Q_TXBD_DESA_L 0x2310
#define REG_HI10Q_TXBD_DESA_H 0x2314
#define REG_HI11Q_TXBD_DESA_L 0x2318
#define REG_HI11Q_TXBD_DESA_H 0x231C
#define REG_HI12Q_TXBD_DESA_L 0x2320
#define REG_HI12Q_TXBD_DESA_H 0x2324
#define REG_HI13Q_TXBD_DESA_L 0x2328

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_H2CQ_TXBD_IDX_V1 0x232C

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_HI13Q_TXBD_DESA_H 0x232C
#define REG_HI14Q_TXBD_DESA_L 0x2330
#define REG_HI14Q_TXBD_DESA_H 0x2334
#define REG_HI15Q_TXBD_DESA_L 0x2338
#define REG_HI15Q_TXBD_DESA_H 0x233C
#define REG_HI16Q_TXBD_DESA_L 0x2340
#define REG_HI16Q_TXBD_DESA_H 0x2344
#define REG_HI17Q_TXBD_DESA_L 0x2348
#define REG_HI17Q_TXBD_DESA_H 0x234C
#define REG_HI18Q_TXBD_DESA_L 0x2350
#define REG_HI18Q_TXBD_DESA_H 0x2354
#define REG_HI19Q_TXBD_DESA_L 0x2358
#define REG_HI19Q_TXBD_DESA_H 0x235C
#define REG_BD_RWPTR_CLR6 0x2364
#define REG_P0HI16Q_TXBD_IDX 0x2370
#define REG_P0HI17Q_TXBD_IDX 0x2374
#define REG_P0HI18Q_TXBD_IDX 0x2378
#define REG_P0HI19Q_TXBD_IDX 0x237C
#define REG_P0HI16Q_HI17Q_TXBD_NUM 0x2380
#define REG_P0HI18Q_HI19Q_TXBD_NUM 0x2384

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_HISR2_V1 0x23B4

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_HISR0 0x23B4

#endif

#if (HALMAC_8812F_SUPPORT || HALMAC_8822C_SUPPORT)

#define REG_PCIE_HISR3_V1 0x23BC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_PCIE_HISR1 0x23BC
#define REG_P0HI8Q_HI9Q_TXBD_NUM 0x23C0
#define REG_P0HI10Q_HI11Q_TXBD_NUM 0x23C4
#define REG_P0HI12Q_HI13Q_TXBD_NUM 0x23C8
#define REG_P0HI14Q_HI15Q_TXBD_NUM 0x23CC
#define REG_ACH6_ACH7_TXBD_NUM 0x23F0

#endif

#if (HALMAC_8192F_SUPPORT)

#define REG_BF0_TIME_SETTING_V1 0x2428
#define REG_BF1_TIME_SETTING_V1 0x242C
#define REG_BF_TIMEOUT_EN_V1 0x2430
#define REG_MACID_RELEASE0_V1 0x2434
#define REG_MACID_RELEASE1_V1 0x2438
#define REG_MACID_RELEASE2_V1 0x243C
#define REG_MACID_RELEASE3_V1 0x2440
#define REG_MACID_RELEASE_SETTING_V1 0x2444
#define REG_FAST_EDCA_VOVI_SETTING_V1 0x2448
#define REG_FAST_EDCA_BEBK_SETTING_V1 0x244C
#define REG_R_MACID_RELEASE_SUCCESS_0_V1 0x2460
#define REG_R_MACID_RELEASE_SUCCESS_1_V1 0x2464
#define REG_R_MACID_RELEASE_SUCCESS_2_V1 0x2468
#define REG_R_MACID_RELEASE_SUCCESS_3_V1 0x246C
#define REG_R_MACID_RELEASE_SUCCESS_CLEAR_OFFSET_V1 0x247C
#define REG_NAN_INFO0 0x2480
#define REG_NAN_INFO1 0x2484
#define REG_NAN_INFO2 0x2488
#define REG_NAN_INFO3 0x248C
#define REG_NAN_INFO4 0x2490
#define REG_NAN_INFO5 0x2494
#define REG_NAN_INFO6 0x2498
#define REG_NAN_INFO7 0x249C
#define REG_NAN_INFO8 0x24A0
#define REG_NAN_INFO9 0x24A4
#define REG_CHNL_INFO_CTRL_V1 0x24D0
#define REG_CHNL_IDLE_TIME_V1 0x24D4
#define REG_CHNL_BUSY_TIME_V1 0x24D8
#define REG_SWPS_CTRL_V1 0x24F4
#define REG_SWPS_PKT_TH_V1 0x24F6
#define REG_SWPS_TIME_TH_V1 0x24F8
#define REG_MACID_SWPS_EN_V1 0x24FC

#endif

#if (HALMAC_8814B_SUPPORT)

#define REG_TXPAGE_INT_CTRL_0 0x3200
#define REG_TXPAGE_INT_CTRL_1 0x3204
#define REG_TXPAGE_INT_CTRL_2 0x3208
#define REG_TXPAGE_INT_CTRL_3 0x320C
#define REG_TXPAGE_INT_CTRL_4 0x3210
#define REG_TXPAGE_INT_CTRL_5 0x3214
#define REG_TXPAGE_INT_CTRL_6 0x3218
#define REG_TXPAGE_INT_CTRL_7 0x321C
#define REG_TXPAGE_INT_CTRL_8 0x3220
#define REG_TXPAGE_INT_CTRL_9 0x3224
#define REG_TXPAGE_INT_CTRL_10 0x3228
#define REG_TXPAGE_INT_CTRL_11 0x322C
#define REG_TXPAGE_INT_CTRL_12 0x3230
#define REG_TXPAGE_INT_CTRL_13 0x3234
#define REG_TXPAGE_INT_CTRL_14 0x3238
#define REG_TXPAGE_INT_CTRL_15 0x323C
#define REG_TXPAGE_INT_CTRL_16 0x3240
#define REG_ACH4_TXBD_IDX 0x3340
#define REG_ACH5_TXBD_IDX 0x3344
#define REG_ACH6_TXBD_IDX 0x3348
#define REG_ACH7_TXBD_IDX 0x334C
#define REG_ACH8_TXBD_IDX 0x3350
#define REG_ACH9_TXBD_IDX 0x3354
#define REG_ACH10_TXBD_IDX 0x3358
#define REG_ACH11_TXBD_IDX 0x335C
#define REG_ACH12_TXBD_IDX 0x3360
#define REG_ACH13_TXBD_IDX 0x3364
#define REG_AC_CHANNEL0_WEIGHT 0x3368
#define REG_AC_CHANNEL1_WEIGHT 0x3369
#define REG_AC_CHANNEL2_WEIGHT 0x336A
#define REG_AC_CHANNEL3_WEIGHT 0x336B
#define REG_AC_CHANNEL4_WEIGHT 0x336C
#define REG_AC_CHANNEL5_WEIGHT 0x336D
#define REG_AC_CHANNEL6_WEIGHT 0x336E
#define REG_AC_CHANNEL7_WEIGHT 0x336F
#define REG_AC_CHANNEL8_WEIGHT 0x3370
#define REG_AC_CHANNEL9_WEIGHT 0x3371
#define REG_AC_CHANNEL10_WEIGHT 0x3372
#define REG_AC_CHANNEL11_WEIGHT 0x3373
#define REG_AC_CHANNEL12_WEIGHT 0x3374
#define REG_AC_CHANNEL13_WEIGHT 0x3375
#define REG_PCIE_HISR2 0x33B4
#define REG_PCIE_HISR3 0x33BC

#endif

/* ----------------------------------------------------- */
/*	*/
/* 0xFB00h ~ 0xFCFFh	TX/RX packet buffer affress */
/*	*/
/* ----------------------------------------------------- */
#define REG_RXPKTBUF_STARTADDR 0xFB00
#define REG_TXPKTBUF_STARTADDR 0xFC00

/* ----------------------------------------------------- */
/*	*/
/* 0xFD00h ~ 0xFDFFh	8051 CPU Local REG */
/*	*/
/* ----------------------------------------------------- */
#define REG_SYS_CTRL 0xFD00
#define REG_PONSTS_RPT1 0xFD01
#define REG_PONSTS_RPT2 0xFD02
#define REG_PONSTS_RPT3 0xFD03
#define REG_PONSTS_RPT4 0xFD04 /* 0x84 */
#define REG_PONSTS_RPT5 0xFD05 /* 0x85 */
#define REG_8051ERRFLAG 0xFD08
#define REG_8051ERRFLAG_MASK 0xFD09
#define REG_TXADDRH 0xFD10 /* Tx Packet High Address */
#define REG_RXADDRH 0xFD11 /* Rx Packet High Address */
#define REG_TXADDRH_EXT 0xFD12

#define REG_U3_STATE 0xFD48

/* for MAILBOX */
#define REG_OUTDATA0 0xFD50
#define REG_OUTDATA1 0xFD54
#define REG_OUTRDY 0xFD58 /* bit[0] : OutReady, bit[1] : OutEmptyIntEn */

#define REG_INDATA0 0xFD60
#define REG_INDATA1 0xFD64
#define REG_INRDY 0xFD68 /* bit[0] : InReady, bit[1] : InRdyIntEn */

/* MCU ERROR debug REG */
#define REG_MCUERR_PCLSB 0xFD90 /* PC[7:0] */
#define REG_MCUERR_PCMSB 0xFD91 /* PC[15:8] */
#define REG_MCUERR_ACC 0xFD92
#define REG_MCUERR_B 0xFD93
#define REG_MCUERR_DPTRLSB 0xFD94 /* DPTR[7:0] */
#define REG_MCUERR_DPTRMSB 0xFD95 /* DPTR[15:8] */
#define REG_MCUERR_SP 0xFD96 /* SP[7:0] */
#define REG_MCUERR_IE 0xFD97 /* IE[7:0] */
#define REG_MCUERR_EIE 0xFD98 /* EIE[7:0] */
#define REG_VERA_SIM 0xFD9F
/* 0xFD99~0xFD9F are reserved.. */

/* ----------------------------------------------------- */
/*	*/
/* 0xFE00h ~ 0xFEFFh	USB Configuration */
/*	*/
/* ----------------------------------------------------- */

/* RTS5101 USB Register Definition */
#define REG_USB_SETUP_DEC_INT 0xFE00
#define REG_USB_DMACTL 0xFE01
#define REG_USB_IRQSTAT0 0xFE02
#define REG_USB_IRQSTAT1 0xFE03
#define REG_USB_IRQEN0 0xFE04
#define REG_USB_IRQEN1 0xFE05
#define REG_USB_AUTOPTRL 0xFE06
#define REG_USB_AUTOPTRH 0xFE07
#define REG_USB_AUTODAT 0xFE08

#define REG_USB_SCRATCH0 0xFE09
#define REG_USB_SCRATCH1 0xFE0A
#define REG_USB_SEEPROM 0xFE0B
#define REG_USB_GPIO0 0xFE0C
#define REG_USB_GPIO0DIR 0xFE0D
#define REG_USB_CLKSEL 0xFE0E
#define REG_USB_BOOTCTL 0xFE0F

#define REG_USB_USBCTL 0xFE10
#define REG_USB_USBSTAT 0xFE11
#define REG_USB_DEVADDR 0xFE12
#define REG_USB_USBTEST 0xFE13
#define REG_USB_FNUM0 0xFE14
#define REG_USB_FNUM1 0xFE15

#define REG_USB_EP_IDX 0xFE20
#define REG_USB_EP_CFG 0xFE21
#define REG_USB_EP_CTL 0xFE22
#define REG_USB_EP_STAT 0xFE23
#define REG_USB_EP_IRQ 0xFE24
#define REG_USB_EP_IRQEN 0xFE25
#define REG_USB_EP_MAXPKT0 0xFE26
#define REG_USB_EP_MAXPKT1 0xFE27
#define REG_USB_EP_DAT 0xFE28
#define REG_USB_EP_BC0 0xFE29
#define REG_USB_EP_BC1 0xFE2A
#define REG_USB_EP_TC0 0xFE2B
#define REG_USB_EP_TC1 0xFE2C
#define REG_USB_EP_TC2 0xFE2D
#define REG_USB_EP_CTL2 0xFE2E

#define REG_USB_INFO 0xFE17
#define REG_USB_SPECIAL_OPTION 0xFE55
#define REG_USB_DMA_AGG_TO 0xFE5B
#define REG_USB_AGG_TO 0xFE5C
#define REG_USB_AGG_TH 0xFE5D

#define REG_USB_VID 0xFE60
#define REG_USB_PID 0xFE62
#define REG_USB_OPT 0xFE64
#define REG_USB_CONFIG 0xFE65

#define REG_USB_PHY_PARA1 0xFE68
#define REG_USB_PHY_PARA2 0xFE69
#define REG_USB_PHY_PARA3 0xFE6A
#define REG_USB_PHY_PARA4 0xFE6B
#define REG_USB_OPT2 0xFE6C
#define REG_USB_MAC_ADDR 0xFE70
#define REG_USB_MANUFACTURE_SETTING 0xFE80
#define REG_USB_PRODUCT_STRING 0xFEA0
#define REG_USB_SERIAL_NUMBER_STRING 0xFED0

#define REG_USB_ALTERNATE_SETTING 0xFE4F
#define REG_USB_INT_BINTERVAL 0xFE6E
#define REG_USB_GPS_EP_CONFIG 0xFE6D

#endif /* __HALMAC_COM_REG_H__ */
